unit ucuEmprestarLivro;


interface

uses
  utuStateMachine, utuRequest, ucuUseCase, acuFramework, nguCliente;

type
  ucEmprestarLivro = class(ucUseCase)
  public
    procedure Initialize; Override;
    procedure Finalize; Override;
  private
    //fCliente : ngCliente;
    //fListaEmprestimos : acPersistentObjectList;
  published
    procedure Effect_PesquisarUsuario(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_EscolherUsuario(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_PesquisarExemplar(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_CancelarEscolhaExemplar(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_ConcluirEdicaoEmprestimo(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_RemoverExemplar(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_ConfirmarEmprestimo(piRequest: utRequest; piTransition: utTransition);
    procedure Effect_CancelarEmprestimo(piRequest: utRequest; piTransition: utTransition);
  end;

implementation

uses
  ucuManager,
  utuMessage {define utField},
  utuEmprestimoOQL,
  acuObject {define acEnumerator},
  nguExemplar,

  nguLivro,

  nguSituacaoExemplar;

procedure ucEmprestarLivro.Initialize;
begin
  inherited;
end;

procedure ucEmprestarLivro.Finalize;
begin
  
  inherited;
end;

procedure ucEmprestarLivro.Effect_PesquisarUsuario(piRequest: utRequest; piTransition: utTransition);
var
  lFieldUsuario,
  lFieldUsuarios: utField;
  lOQL: IObterClientes;
  lListUsuarios: acPersistentObjectList;
  lEnumUsuario: acEnumerator;
  lUsuario: ngCliente;
begin
  lFieldUsuario := piRequest.Message.RootField.FieldByName('Cliente');

  lOQL := utEmprestimoOQL.ObterClientes(Self.Session);

  lFieldUsuarios := piRequest.Response.RootField.AddField('Clientes');

  lListUsuarios := acPersistentObjectList.Create(ngCliente);
  try

    if lFieldUsuario.HasAttribute('Nome')
      then lOQL.Param_nome.Value := lFieldUsuario.AttributeByName('Nome').AsString
      else lOQL.Param_nome.SetNull;

    if lFieldUsuario.HasAttribute('CPF')
      then lOQL.Param_cpf.Value := lFieldUsuario.AttributeByName('CPF').AsString
      else lOQL.Param_cpf.SetNull;

    lOQL.LoadList(lListUsuarios);

    lEnumUsuario := acEnumerator.Create(lListUsuarios);
    try
      while not lEnumUsuario.EOL do
      begin
        lUsuario := lEnumUsuario.Current as ngCliente;
        lUsuario.toXML(lFieldUsuarios.AddField('Cliente'));
        lEnumUsuario.MoveNext;
      end;
    finally
      lEnumUsuario.Free;
    end;


  finally
    lListUsuarios.Free;
  end;

  piRequest.Processed := True;
end;

procedure ucEmprestarLivro.Effect_EscolherUsuario(piRequest: utRequest; piTransition: utTransition);
var
lFieldCliente : utField;
lCliente : ngCliente;
begin
  lFieldCliente := piRequest.Message.RootField.FieldByName('Cliente');

  try
    lCliente := Self.Session.LoadObject(ngCliente,acOIDInt.Create(lFieldCliente.AttributeByName('OID').AsInteger)) as ngCliente;
  except
    on e:acObjectNotFoundException do
      raise EucException.Create('Cliente no encontrado.');
    else
      raise EucException.Create('No foi possvel buscar o cliente.');
  end;

  //if not Assigned(lCliente)  then raise EucException.Create('Cliente no encontrado.');

  lCliente.toXML(piRequest.Response.RootField.AddField('Cliente'));

  piRequest.Processed := True;
end;

procedure ucEmprestarLivro.Effect_PesquisarExemplar(piRequest: utRequest; piTransition: utTransition);
var
  lFieldExemplar,
  lFieldExemplares,
  lFieldLivro,
  lFieldLivros,
  lFieldSituacao: utField;
  lOQL: IObterExemplares;
  lListExemplares: acPersistentObjectList;
  lEnumExemplares: acEnumerator;
  lExemplar: ngExemplar;
  lLivro,lUltimoLivro: ngLivro;
  lSituacao: ngSituacaoExemplar;
begin
  lFieldExemplar := piRequest.Message.RootField.FieldByName('Exemplar');
  //if lFieldExemplar.HasField('Livro')
  //then lFieldLivro :=  lFieldExemplar.FieldByName('Livro');

  lOQL := utEmprestimoOQL.ObterExemplares(Self.Session);

  lFieldLivros := piRequest.Response.RootField.AddField('Livros');

  lListExemplares := acPersistentObjectList.Create(ngExemplar); {Lista de exemplar obtida}
  {Comeamos a formatao da resposta}
  try

    {Colocamos os parmetros para a busca oql}
    if lFieldExemplar.HasAttribute('Codigo')
      then lOQL.Param_codigo.Value := lFieldExemplar.AttributeByName('Codigo').AsString
      else lOQL.Param_codigo.SetNull;

    {Se o xml tinha livro...}
    if lFieldExemplar.HasField('Livro') then
    begin
      if lFieldExemplar.FieldByName('Livro').HasAttribute('Titulo')
        then lOQL.Param_titulo.Value := lFieldExemplar.FieldByName('Livro').AttributeByName('Titulo').AsString
        else lOQL.Param_titulo.SetNull;
    end
    else lOQL.Param_titulo.SetNull;

    lOQL.LoadList(lListExemplares);

    lEnumExemplares := acEnumerator.Create(lListExemplares);
    try
      lExemplar := lEnumExemplares.Current as ngExemplar;
      lLivro := lExemplar.Livro.Partner as ngLivro;
      
      {Guardamos ultimo livro para agrup-los e criamos seu campo exemplares}
      lUltimoLivro := lLivro;
      lFieldLivro := lFieldLivros.AddField('Livro');
      lLivro.toXML(lFieldLivro);
      lFieldExemplares := lFieldLivro.AddField('Exemplares');

      while not lEnumExemplares.EOL do
      begin
        lExemplar := lEnumExemplares.Current as ngExemplar;
        lLivro := lExemplar.Livro.Partner as ngLivro;
        lSituacao := lExemplar.Situacao.Partner as ngSituacaoExemplar;
        
        if not lLivro.Equals(lUltimoLivro) then
        begin
          lUltimoLivro := lLivro;
          lFieldLivro := lFieldLivros.AddField('Livro');
          lLivro.toXML(lFieldLivro);
          lFieldExemplares := lFieldLivro.AddField('Exemplares');
        end;

        {Criamos as tags no xml}
        lFieldExemplar := lFieldExemplares.AddField('Exemplar');
        lFieldSituacao := lFieldExemplar.AddField('Situacao');

        {Damos os valores as tags}
        lExemplar.toXML(lFieldExemplar);
        lSituacao.toXML(lFieldSituacao);

        lEnumExemplares.MoveNext;
      end;
    finally
      lEnumExemplares.Free;
    end;

  finally
    lListExemplares.Free;
  end;

  piRequest.Processed := True;
end;

procedure ucEmprestarLivro.Effect_CancelarEscolhaExemplar(piRequest: utRequest; piTransition: utTransition);
begin


  piRequest.Processed := True;
end;

procedure ucEmprestarLivro.Effect_ConcluirEdicaoEmprestimo(piRequest: utRequest; piTransition: utTransition);
begin
  
  
  piRequest.Processed := True;
end;

procedure ucEmprestarLivro.Effect_RemoverExemplar(piRequest: utRequest; piTransition: utTransition);
begin
  
  
  piRequest.Processed := True;
end;

procedure ucEmprestarLivro.Effect_ConfirmarEmprestimo(piRequest: utRequest; piTransition: utTransition);
begin
  
  
  piRequest.Processed := True;
end;

procedure ucEmprestarLivro.Effect_CancelarEmprestimo(piRequest: utRequest; piTransition: utTransition);
begin
  
  
  piRequest.Processed := True;
end;


initialization

  gucManager.RegisterClass(3219,ucEmprestarLivro);

end.
